;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Fireball sprite
;;       by
;;   wiiqwertyuiop
;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;

Table: 
db $15,$EA		; Speed table (right,left)

XDISP:
db $00,$10
db $00,$10
;Facing other direction
db $10,$00
db $10,$00

TILEMAP:
db $4A,$4C		; Frame 1
db $6A,$6C		; Frame 2

PROPERTIES: 		  
db $75,$35		; YXPPCCCT format (right,left)  

!AnimationSpeed = #3	; Animation speed
!Frames = $01		; Number of frames. MUST BE A POWER OF 2!

;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; INIT
;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
JSR SUBHORZPOS
TYA
STA $C2,x
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN
;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR SpriteCode
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Sprite Code Below
;;;;;;;;;;;;;;;;;;;;;;;;;;

	
SpriteCode:
JSR Graphics			; Draw the sprite
LDA $9D
BNE Return
LDY $C2,x
LDA Table,y
STA $B6,x
JSL $018022			; Update X position without gravity
JSL $01A7DC			; Interact with Mario
BCC Return
LDA $1490
BNE HasStar
JSL $00F5B7
Return:
RTS

HasStar:
JSR DrawSmoke
STZ $14C8,x
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GFX Below
;;;;;;;;;;;;;;;;;;;;;;;;;;

Graphics:
	JSR GET_DRAW_INFO 

	LDA $14			;\ Frame counter ..
	LSR !AnimationSpeed ;A	; |
	;LSR A			; | Add in frame animation rate; More LSRs for slower animation.
	AND #!Frames		; | 01 means we animate between 2 frames (00 and 01).
	ASL A 			; | 
	;ASL A			; | ASL x2 (0-4) makes it switch between the first byte and fifth byte,
	STA $03 		;/ i.e. first animation and second animation. The result is stored into $03.

	LDA $C2,x
	STA $02			; Store direction to $02 to use later on

NoAdd:	
	PHX			;\ Push the sprite index, since we're using it for a loop.	
	LDX #$01		;/ X = number of times to loop through. Since we only draw one MORE tile, loop one more time.
Loop:
	PHX			; Push number of tiles to loop through.
	TXA			;\
	ORA $03			;/ Transfer it to X and add in the "left displacement" if necessary.
	TAX			
	
	PHX
	LDA $02
	BNE FacingLeft
	INX #4
	FacingLeft:
	LDA $00			;\
	CLC			; |
	ADC XDISP,x		; |
	STA $0300,y		;/ Draw the X position of the sprite
	PLX

SHARE:
	LDA $01			;\
	;CLC			; |
	;ADC #$10		; |
	STA $0301,y		;/ Draw the Y position of the sprite

	LDA TILEMAP,x		
	STA $0302,y		

	PHX			; Push number of times to go through loop + "left" displacement if necessary.
	LDX $02			;\
	LDA PROPERTIES,x	; | Set properties based on direction.
	STA $0303,y		;/
	PLX			; Pull number of times to go through loop.

	INY			
	INY			
	INY			
	INY			

	PLX
	DEX

	BPL Loop
	
	PLX

	LDY #$02		
	LDA #$01		
				

	JSL $01B7B3		; Call the routine that draws the sprite.
	RTS			
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:             db $0C,$1C
SPR_T2:             db $01,$02

GET_DRAW_INFO:      STZ $186C,x             ; reset sprite offscreen flag, vertical
	STZ $15A0,x             ; reset sprite offscreen flag, horizontal
	LDA $E4,x               ; \
	CMP $1A                 ;  | set horizontal offscreen if necessary
	LDA $14E0,x             ;  |
	SBC $1B                 ;  |
	BEQ ON_SCREEN_X         ;  |
	INC $15A0,x             ; /

ON_SCREEN_X:        LDA $14E0,x             ; \
	XBA	 ;  |
	LDA $E4,x               ;  |
	REP #$20                ;  |
	SEC	 ;  |
	SBC $1A                 ;  | mark sprite invalid if far enough off screen
	CLC	 ;  |
	ADC.w #$0040            ;  |
	CMP.w #$0180            ;  |
	SEP #$20                ;  |
	ROL A                   ;  |
	AND #$01                ;  |
	STA $15C4,x             ; / 
	BNE INVALID             ; 
	
	LDY #$00                ; \ set up loop:
	LDA $1662,x             ;  | 
	AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
	BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
	INY	 ; / 
ON_SCREEN_LOOP:     LDA $D8,x               ; \ 
	CLC	 ;  | set vertical offscreen if necessary
	ADC SPR_T1,y            ;  |
	PHP	 ;  |
	CMP $1C                 ;  | (vert screen boundry)
	ROL $00                 ;  |
	PLP	 ;  |
	LDA $14D4,x             ;  | 
	ADC #$00                ;  |
	LSR $00                 ;  |
	SBC $1D                 ;  |
	BEQ ON_SCREEN_Y         ;  |
	LDA $186C,x             ;  | (vert offscreen)
	ORA SPR_T2,y            ;  |
	STA $186C,x             ;  |
ON_SCREEN_Y:        DEY	 ;  |
	BPL ON_SCREEN_LOOP      ; /

	LDY $15EA,x             ; get offset to sprite OAM
	LDA $E4,x               ; \ 
	SEC	 ;  | 
	SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
	STA $00                 ; / 
	LDA $D8,x               ; \ 
	SEC	 ;  | 
	SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
	STA $01                 ; / 
	RTS	 ; return

INVALID:            PLA	 ; \ return from *main gfx routine* subroutine...
	PLA	 ;  |    ...(not just this subroutine)
	RTS	 ; /



SUBHORZPOS:				LDY #$00
					LDA $94	
					SEC                 
					SBC $E4,x				
					STA $0F					
					LDA $95					
					SBC $14E0,x				
					BPL SPR_L16            
					INY                     
SPR_L16:				RTS                     

DrawSmoke:
	LDY #$03
-
	LDA $17C0,y
	BEQ +
	DEY
	BPL -
	RTS
+
	LDA #$01
	STA $17C0,y
	LDA #$1C     
	STA $17CC,y
	LDA $D8,x
	STA $17C4,y
	LDA $E4,x
	STA $17C8,y

	LDY #$03
-
	LDA $17C0,y
	BEQ +
	DEY
	BPL -
	RTS
+
	LDA #$01
	STA $17C0,y
	LDA #$1C     
	STA $17CC,y
	LDA $D8,x
	STA $17C4,y
	LDA $E4,x
	CLC
	ADC #$10
	STA $17C8,y
	RTS